@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2010 Junjiro R. Okajima
+ * Copyright (C) 2005-2011 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,8 +52,7 @@ static int h_permission(struct inode *h_inode, int mask,
 		&& write_mask && !(mask & MAY_READ))
 	    || !h_inode->i_op->permission) {
 		/* AuLabel(generic_permission); */
-		err = generic_permission(h_inode, mask,
-					 h_inode->i_op->check_acl);
+		err = generic_permission(h_inode, mask);
 	} else {
 		/* AuLabel(h_inode->permission); */
 		err = h_inode->i_op->permission(h_inode, mask);
@@ -92,6 +91,10 @@ static int aufs_permission(struct inode *inode, int mask)
 	struct super_block *sb;
 	struct au_branch *br;
 
+	/* todo: support rcu-walk? */
+	if (mask & MAY_NOT_BLOCK)
+		return -ECHILD;
+
 	sb = inode->i_sb;
 	si_read_lock(sb, AuLock_FLUSH);
 	ii_read_lock_child(inode);
@@ -157,7 +160,7 @@ static struct dentry *aufs_lookup(struct inode *dir, struct dentry *dentry,
 	struct dentry *ret, *parent;
 	struct inode *inode;
 	struct super_block *sb;
-	int err, npositive;
+	int err, npositive, lc_idx;
 
 	IMustLock(dir);
 
@@ -175,35 +178,54 @@ static struct dentry *aufs_lookup(struct inode *dir, struct dentry *dentry,
 	if (unlikely(err))
 		goto out_si;
 
-	npositive = -EIO;
+	inode = NULL;
+	npositive = 0; /* suppress a warning */
 	parent = dentry->d_parent; /* dir inode is locked */
 	di_read_lock_parent(parent, AuLock_IR);
-	err = au_digen_test(parent, au_sigen(sb));
+	err = au_alive_dir(parent);
 	if (!err)
+		err = au_digen_test(parent, au_sigen(sb));
+	if (!err) {
 		npositive = au_lkup_dentry(dentry, au_dbstart(parent),
 					   /*type*/0, nd);
+		err = npositive;
+	}
 	di_read_unlock(parent, AuLock_IR);
-	err = npositive;
 	ret = ERR_PTR(err);
 	if (unlikely(err < 0))
 		goto out_unlock;
 
-	inode = NULL;
 	if (npositive) {
 		inode = au_new_inode(dentry, /*must_new*/0);
 		ret = (void *)inode;
 	}
-	if (IS_ERR(inode))
+	if (IS_ERR(inode)) {
+		inode = NULL;
 		goto out_unlock;
+	}
 
 	ret = d_splice_alias(inode, dentry);
 	if (unlikely(IS_ERR(ret) && inode)) {
 		ii_write_unlock(inode);
+		lc_idx = AuLcNonDir_IIINFO;
+		if (S_ISLNK(inode->i_mode))
+			lc_idx = AuLcSymlink_IIINFO;
+		else if (S_ISDIR(inode->i_mode))
+			lc_idx = AuLcDir_IIINFO;
+		au_rw_class(&au_ii(inode)->ii_rwsem, au_lc_key + lc_idx);
 		iput(inode);
 	}
 
 out_unlock:
 	di_write_unlock(dentry);
+	if (unlikely(IS_ERR(ret) && inode)) {
+		lc_idx = AuLcNonDir_DIINFO;
+		if (S_ISLNK(inode->i_mode))
+			lc_idx = AuLcSymlink_DIINFO;
+		else if (S_ISDIR(inode->i_mode))
+			lc_idx = AuLcDir_DIINFO;
+		au_rw_class(&au_di(dentry)->di_rwsem, au_lc_key + lc_idx);
+	}
 out_si:
 	si_read_unlock(sb);
 out:
@@ -563,7 +585,7 @@ static int au_pin_and_icpup(struct dentry *dentry, struct iattr *ia,
 
 	h_file = NULL;
 	hi_wh = NULL;
-	if (au_ftest_icpup(a->flags, DID_CPUP) && au_d_removed(dentry)) {
+	if (au_ftest_icpup(a->flags, DID_CPUP) && d_unlinked(dentry)) {
 		hi_wh = au_hi_wh(inode, a->btgt);
 		if (!hi_wh) {
 			err = au_sio_cpup_wh(dentry, a->btgt, sz, /*file*/NULL);
@@ -654,10 +676,10 @@ static int aufs_setattr(struct dentry *dentry, struct iattr *ia)
 	} else {
 		/* fchmod() doesn't pass ia_file */
 		a->udba = au_opt_udba(sb);
-		/* no au_d_removed(), to set UDBA_NONE for root */
+		di_write_lock_child(dentry);
+		/* no d_unlinked(), to set UDBA_NONE for root */
 		if (d_unhashed(dentry))
 			a->udba = AuOpt_UDBA_NONE;
-		di_write_lock_child(dentry);
 		if (a->udba != AuOpt_UDBA_NONE) {
 			AuDebugOn(IS_ROOT(dentry));
 			err = au_reval_for_attr(dentry, au_sigen(sb));
@@ -770,7 +792,7 @@ static int aufs_getattr(struct vfsmount *mnt __maybe_unused,
 	udba_none = !!au_opt_test(mnt_flags, UDBA_NONE);
 
 	/* support fstat(2) */
-	if (!au_d_removed(dentry) && !udba_none) {
+	if (!d_unlinked(dentry) && !udba_none) {
 		unsigned int sigen = au_sigen(sb);
 		err = au_digen_test(dentry, sigen);
 		if (!err) {
