@@ -1,12 +1,11 @@
 /*
  * kernel/power/tuxonice_modules.c
  *
- * Copyright (C) 2004-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2004-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  */
 
 #include <linux/suspend.h>
-#include <linux/module.h>
 #include "tuxonice.h"
 #include "tuxonice_modules.h"
 #include "tuxonice_sysfs.h"
@@ -14,10 +13,14 @@
 
 LIST_HEAD(toi_filters);
 LIST_HEAD(toiAllocators);
+
 LIST_HEAD(toi_modules);
+EXPORT_SYMBOL_GPL(toi_modules);
 
 struct toi_module_ops *toiActiveAllocator;
-int toi_num_filters;
+EXPORT_SYMBOL_GPL(toiActiveAllocator);
+
+static int toi_num_filters;
 int toiNumAllocators, toi_num_modules;
 
 /*
@@ -28,7 +31,7 @@ int toiNumAllocators, toi_num_modules;
  * kernel. We can exclude data for pageset2 because it will be
  * available anyway once the kernel is copied back.
  */
-int toi_header_storage_for_modules(void)
+long toi_header_storage_for_modules(void)
 {
 	struct toi_module_ops *this_module;
 	int bytes = 0;
@@ -51,6 +54,37 @@ int toi_header_storage_for_modules(void)
 	return bytes + sizeof(struct toi_module_header);
 }
 
+void print_toi_header_storage_for_modules(void)
+{
+	struct toi_module_ops *this_module;
+	int bytes = 0;
+
+	printk(KERN_DEBUG "Header storage:\n");
+	list_for_each_entry(this_module, &toi_modules, module_list) {
+		if (!this_module->enabled ||
+		    (this_module->type == WRITER_MODULE &&
+		     toiActiveAllocator != this_module))
+			continue;
+		if (this_module->storage_needed) {
+			int this = this_module->storage_needed() +
+				sizeof(struct toi_module_header) +
+				sizeof(int);
+			this_module->header_requested = this;
+			bytes += this;
+			printk(KERN_DEBUG "+ %16s : %-4d/%d.\n",
+					this_module->name,
+					this_module->header_used, this);
+		}
+	}
+
+	printk(KERN_DEBUG "+ empty terminator : %zu.\n",
+			sizeof(struct toi_module_header));
+	printk(KERN_DEBUG "                     ====\n");
+	printk(KERN_DEBUG "                     %zu\n",
+			bytes + sizeof(struct toi_module_header));
+}
+EXPORT_SYMBOL_GPL(print_toi_header_storage_for_modules);
+
 /*
  * toi_memory_for_modules
  *
@@ -58,9 +92,9 @@ int toi_header_storage_for_modules(void)
  * doing their work during the cycle.
  */
 
-int toi_memory_for_modules(int print_parts)
+long toi_memory_for_modules(int print_parts)
 {
-	int bytes = 0, result;
+	long bytes = 0, result;
 	struct toi_module_ops *this_module;
 
 	if (print_parts)
@@ -72,17 +106,17 @@ int toi_memory_for_modules(int print_parts)
 		if (this_module->memory_needed) {
 			this = this_module->memory_needed();
 			if (print_parts)
-				printk(KERN_INFO "%10d bytes (%5d pages) for "
-						"module '%s'.\n",
-						this, this >> PAGE_SHIFT,
+				printk(KERN_INFO "%10d bytes (%5ld pages) for "
+						"module '%s'.\n", this,
+						DIV_ROUND_UP(this, PAGE_SIZE),
 						this_module->name);
 			bytes += this;
 		}
 	}
 
-	result = ((bytes + PAGE_SIZE - 1) >> PAGE_SHIFT);
+	result = DIV_ROUND_UP(bytes, PAGE_SIZE);
 	if (print_parts)
-		printk(KERN_INFO " => %d bytes, %d pages.\n", bytes, result);
+		printk(KERN_INFO " => %ld bytes, %ld pages.\n", bytes, result);
 
 	return result;
 }
@@ -203,16 +237,18 @@ int toi_register_module(struct toi_module_ops *module)
 		break;
 	case MISC_MODULE:
 	case MISC_HIDDEN_MODULE:
+	case BIO_ALLOCATOR_MODULE:
 		break;
 	default:
-		printk("Hmmm. Module '%s' has an invalid type."
+		printk(KERN_ERR "Hmmm. Module '%s' has an invalid type."
 			" It has been ignored.\n", module->name);
 		return -EINVAL;
 	}
 	list_add_tail(&module->module_list, &toi_modules);
 	toi_num_modules++;
 
-	if (!module->directory && !module->shared_directory)
+	if ((!module->directory && !module->shared_directory) ||
+			!module->sysfs_data || !module->num_sysfs_entries)
 		return 0;
 
 	/*
@@ -224,8 +260,8 @@ int toi_register_module(struct toi_module_ops *module)
 		struct toi_module_ops *shared =
 			toi_find_module_given_dir(module->shared_directory);
 		if (!shared) {
-			printk("TuxOnIce: Module %s wants to share %s's "
-					"directory but %s isn't loaded.\n",
+			printk(KERN_ERR "TuxOnIce: Module %s wants to share "
+					"%s's directory but %s isn't loaded.\n",
 					module->name, module->shared_directory,
 					module->shared_directory);
 			toi_unregister_module(module);
@@ -234,7 +270,7 @@ int toi_register_module(struct toi_module_ops *module)
 		kobj = shared->dir_kobj;
 	} else {
 		if (!strncmp(module->directory, "[ROOT]", 6))
-			kobj = &toi_subsys.kobj;
+			kobj = tuxonice_kobj;
 		else
 			kobj = make_toi_sysdir(module->directory);
 	}
@@ -247,6 +283,7 @@ int toi_register_module(struct toi_module_ops *module)
 	}
 	return 0;
 }
+EXPORT_SYMBOL_GPL(toi_register_module);
 
 /*
  * toi_unregister_module
@@ -282,15 +319,17 @@ void toi_unregister_module(struct toi_module_ops *module)
 		break;
 	case MISC_MODULE:
 	case MISC_HIDDEN_MODULE:
+	case BIO_ALLOCATOR_MODULE:
 		break;
 	default:
-		printk("Hmmm. Module '%s' has an invalid type."
+		printk(KERN_ERR "Module '%s' has an invalid type."
 			" It has been ignored.\n", module->name);
 		return;
 	}
 	list_del(&module->module_list);
 	toi_num_modules--;
 }
+EXPORT_SYMBOL_GPL(toi_unregister_module);
 
 /*
  * toi_move_module_tail
@@ -310,9 +349,10 @@ void toi_move_module_tail(struct toi_module_ops *module)
 		break;
 	case MISC_MODULE:
 	case MISC_HIDDEN_MODULE:
+	case BIO_ALLOCATOR_MODULE:
 		break;
 	default:
-		printk("Hmmm. Module '%s' has an invalid type."
+		printk(KERN_ERR "Module '%s' has an invalid type."
 			" It has been ignored.\n", module->name);
 		return;
 	}
@@ -338,12 +378,12 @@ int toi_initialise_modules(int starting_cycle, int early)
 		if (this_module->early != early)
 			continue;
 		if (this_module->initialise) {
-			toi_message(TOI_MEMORY, TOI_MEDIUM, 1,
-				"Initialising module %s.\n",
-				this_module->name);
 			result = this_module->initialise(starting_cycle);
-			if (result)
+			if (result) {
+				toi_cleanup_modules(starting_cycle);
 				return result;
+			}
+			this_module->initialised = 1;
 		}
 	}
 
@@ -360,14 +400,41 @@ void toi_cleanup_modules(int finishing_cycle)
 	struct toi_module_ops *this_module;
 
 	list_for_each_entry(this_module, &toi_modules, module_list) {
-		if (!this_module->enabled)
+		if (!this_module->enabled || !this_module->initialised)
 			continue;
-		if (this_module->cleanup) {
-			toi_message(TOI_MEMORY, TOI_MEDIUM, 1,
-				"Cleaning up module %s.\n",
-				this_module->name);
+		if (this_module->cleanup)
 			this_module->cleanup(finishing_cycle);
-		}
+		this_module->initialised = 0;
+	}
+}
+
+/*
+ * toi_pre_atomic_restore_modules
+ *
+ * Get ready to do some work!
+ */
+void toi_pre_atomic_restore_modules(struct toi_boot_kernel_data *bkd)
+{
+	struct toi_module_ops *this_module;
+
+	list_for_each_entry(this_module, &toi_modules, module_list) {
+		if (this_module->enabled && this_module->pre_atomic_restore)
+			this_module->pre_atomic_restore(bkd);
+	}
+}
+
+/*
+ * toi_post_atomic_restore_modules
+ *
+ * Get ready to do some work!
+ */
+void toi_post_atomic_restore_modules(struct toi_boot_kernel_data *bkd)
+{
+	struct toi_module_ops *this_module;
+
+	list_for_each_entry(this_module, &toi_modules, module_list) {
+		if (this_module->enabled && this_module->post_atomic_restore)
+			this_module->post_atomic_restore(bkd);
 	}
 }
 
@@ -390,6 +457,7 @@ struct toi_module_ops *toi_get_next_filter(struct toi_module_ops *filter_sought)
 
 	return toiActiveAllocator;
 }
+EXPORT_SYMBOL_GPL(toi_get_next_filter);
 
 /**
  * toi_show_modules: Printk what support is loaded.
@@ -399,7 +467,7 @@ void toi_print_modules(void)
 	struct toi_module_ops *this_module;
 	int prev = 0;
 
-	printk("TuxOnIce " TOI_CORE_VERSION ", with support for");
+	printk(KERN_INFO "TuxOnIce " TOI_CORE_VERSION ", with support for");
 
 	list_for_each_entry(this_module, &toi_modules, module_list) {
 		if (this_module->type == MISC_HIDDEN_MODULE)
@@ -452,10 +520,3 @@ void toi_put_modules(void)
 	list_for_each_entry(this_module, &toi_modules, module_list)
 		module_put(this_module->module);
 }
-
-#ifdef CONFIG_TOI_EXPORTS
-EXPORT_SYMBOL_GPL(toi_register_module);
-EXPORT_SYMBOL_GPL(toi_unregister_module);
-EXPORT_SYMBOL_GPL(toi_get_next_filter);
-EXPORT_SYMBOL_GPL(toiActiveAllocator);
-#endif
